/*
 * Copyright (c) 2014 by ELECTRONIC ASSEMBLY <technik@lcd-module.de>
 * EA uniTFT-Serie library for arduino.
 *
 * This file is free software; you can redistribute it and/or modify
 * it under the terms of either the GNU General Public License version 2
 * or the GNU Lesser General Public License version 2.1, both as
 * published by the Free Software Foundation.
 */

#ifndef UNITFT_H
#define UNITFT_H

#define DC1 0x11
#define DC2 0x12

#define ACK  0x06
#define NACK 0x15

#define ESC  0x1B

#define REPEAT 3
#define TIMEOUT 			2000
#define TIMEOUTBUFFERFULL	 100

class unitft
{
    using ReceiveEvent = void (*)(byte);
  public:
    void initialize_spi (byte p_cs, byte p_mosi, byte p_miso, byte p_clk, byte p_sbuf); //Use uniTFT with SPI interface
	void initialize_i2c (int freqKHz, byte slaveAddress8bit, byte p_sbuf);				//Use uniTFT with I2C interface
	void RegisterCallbackReceive(ReceiveEvent callbackReceiveData);						//Register function to be called if uniTFT has data to send
	const byte *receivedData() {return (const byte*)receivepacket;}						//Access to received data.
	
	boolean sendDataPacket(const char *dat, unsigned int len);							//Send data to uniTFT

	void isr_sendBuffer(); //retrieves data from uniTFT
	
  private:
    byte p_cs;
	byte p_mosi;
	byte p_miso;
    byte p_clk;
    byte p_sbuf;
	byte p_i2cslaveaddress;
    boolean hardware;
	boolean spi;
	
	byte sendpacket[255];
	volatile byte receivepacket[255];
	unsigned int sendBufferReady;
	unsigned int receiveBufferFree;


	ReceiveEvent reveiveDataCallback;

	void initialize(byte p_sbuf);
	void checkReceiveBufferStatus();
	
	
	
	byte sendPacket(const char *dat, byte len, byte type);
	byte recPacket();
	
	void out(byte *dat, byte len) {(spi)?spi_out(dat,len):i2c_out(dat,len);}
	byte rec(){return (spi)?spi_rec():i2c_rec();}
	boolean rec(byte *buf, int len) {return (spi)?spi_rec(buf, len):i2c_rec(buf, len);}
	
	void spi_initialize(byte cs, byte mosi, byte miso, byte clk);
	void spi_out(byte *dat, byte len);
    byte spi_rec();
	boolean spi_rec(byte *buf, int len);
	
	void i2c_initialize(int freqKHz, byte slaveAddress8bit);
    void i2c_out(byte *dat, byte len);
    byte i2c_rec();
	boolean i2c_rec(byte *buf, int len);
};

extern unitft uniTFT;

#endif
